@@ -55,6 +55,7 @@
 static gboolean applet_factory (PanelApplet *applet, const gchar *iid, gpointer data);
 static gboolean applet_fill (PanelApplet *applet);
 static void cdplayer_load_config(CDPlayerData *cd);
+static void show_error(CDPlayerData *cd);
 static void cdplayer_save_config(CDPlayerData *cd);
 
 static void cdplayer_destroy(GtkWidget * widget, gpointer data);
@@ -88,7 +89,7 @@ static void ref_and_remove(GtkWidget *w);
 static GtkWidget *pack_make_hbox(CDPlayerData* cd);
 static void pack_thing(GtkWidget *box, GtkWidget *w, gboolean expand);
 
-static gboolean cd_try_open(CDPlayerData *cd);
+static gboolean cd_try_open(CDPlayerData *cd, int *errcode);
 static void cd_close (CDPlayerData *cd);
 
 static void cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd);
@@ -215,6 +216,31 @@ applet_fill (PanelApplet *applet)
     return(TRUE);
 }
 
+static void
+show_error (CDPlayerData *cd)
+{
+    static GtkWidget *dialog = NULL;
+    if (dialog) {
+        gtk_window_present (GTK_WINDOW (dialog));
+        return;
+    }
+    dialog = gtk_message_dialog_new (NULL,
+				     GTK_DIALOG_DESTROY_WITH_PARENT,
+				     GTK_MESSAGE_ERROR,
+				     GTK_BUTTONS_OK,
+				     _("You do not have permission to use the CD player."));
+
+    g_signal_connect (dialog, "response",
+		      G_CALLBACK (gtk_widget_destroy),
+		      NULL);
+    g_signal_connect (G_OBJECT(dialog), "destroy",
+                      G_CALLBACK(gtk_widget_destroyed), &dialog);
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+    gtk_window_set_screen (GTK_WINDOW (dialog),
+			   gtk_widget_get_screen (cd->panel.applet));
+    gtk_widget_show (dialog);
+}
+
 static void
 cdplayer_load_config(CDPlayerData *cd)
 {
@@ -238,12 +264,13 @@ cdplayer_destroy(GtkWidget * widget, gpointer data)
 {
     CDPlayerData *cd = data;
     GtkTooltips *tooltips;
+    int err;
 
     if (cd->timeout != 0)
         gtk_timeout_remove(cd->timeout);
     cd->timeout = 0;
     /* Since the applet is being destroyed, stop playing cd */
-    if(cd_try_open(cd))
+    if(cd_try_open(cd, &err))
         cdrom_stop(cd->cdrom_device);
     cd_close (cd);
 
@@ -303,8 +330,8 @@ start_gtcd_cb (BonoboUIComponent *component,
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("There was an error executing '%s' : %s"),
+					 GTK_BUTTONS_OK,
+					 _("There was an error executing '%s': %s"),
 					 "gnome-cd",
 					 error->message);
 
@@ -340,20 +367,22 @@ activate_cb (GtkEntry     *entry,
 	     CDPlayerData *cd)
 {
     gchar *newpath;
+    int err;
     
     newpath = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
     if(newpath && strlen(newpath) > 2 && strcmp(cd->devpath, newpath))
             {
-		if(cd_try_open(cd))
+		if(cd_try_open(cd, &err))
 		    cdrom_stop(cd->cdrom_device);
                 cd_close(cd);
                 cd->devpath = g_strdup(newpath);
-                if (!cd_try_open(cd)) {
+                if (!cd_try_open(cd, &err)) {
                     GtkWidget *dialog;
+
                     dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
                                   		     GTK_MESSAGE_ERROR,
-                                  		     GTK_BUTTONS_CLOSE,
-                                  		     "%s is not a proper device path",
+                                  		     GTK_BUTTONS_OK,
+                                  		     _("%s does not seem to be a CD player"),
                                   		     cd->devpath, NULL);
 
 		    gtk_window_set_screen (GTK_WINDOW (dialog),
@@ -386,6 +415,7 @@ static void
 set_default_device (GtkButton *button, gpointer data)
 {
     CDPlayerData *cd = data;
+    int err;
     GtkWidget *entry = g_object_get_data (G_OBJECT (button), "entry");
     
     if (!strcmp(cd->devpath, DEV_PATH))
@@ -395,7 +425,7 @@ set_default_device (GtkButton *button, gpointer data)
     if (cd->devpath)
         g_free(cd->devpath);
     cd->devpath = g_strdup(DEV_PATH);
-    cd_try_open(cd);
+    cd_try_open(cd, &err);
     gtk_entry_set_text (GTK_ENTRY (entry), cd->devpath);
     cdplayer_save_config(cd);  
 }
@@ -410,7 +440,6 @@ preferences_cb (BonoboUIComponent *component,
     GtkWidget *button;
     GtkWidget *hbox;
     GtkWidget *label;
-    GtkWidget *image;
     GtkWidget *entry;
     gint response;
 
@@ -429,25 +458,23 @@ preferences_cb (BonoboUIComponent *component,
     gtk_window_set_screen (GTK_WINDOW (dialog),
 			   gtk_widget_get_screen (cd->panel.applet));
     gtk_dialog_set_default_response(GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
     box = GTK_DIALOG(dialog)->vbox;
     
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD);
+    hbox = gtk_hbox_new (FALSE, 12);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
     gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    image = gtk_image_new_from_stock(GTK_STOCK_CDROM, GTK_ICON_SIZE_BUTTON);
 
-    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 10);
-    gtk_widget_show(image);
-    set_atk_name_description(image, _("Disc Image"), _("An image of a cd-rom disc"));
-
-    label = gtk_label_new_with_mnemonic(_("Device _Path:"));
-    gtk_misc_set_padding (GTK_MISC (label), GNOME_PAD_SMALL, 0);
+    label = gtk_label_new_with_mnemonic(_("Device _path:"));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
 
     entry = gtk_entry_new();
-    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 4);
+    gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show(entry);
     gtk_entry_set_text(GTK_ENTRY(entry), cd->devpath);
     set_atk_name_description(entry, _("Device Path"), _("Set the device path here"));
@@ -721,11 +748,10 @@ pack_thing(GtkWidget *box, GtkWidget *w, gboolean expand)
 
 /* Deal with the hardware stuff */
 static gboolean
-cd_try_open(CDPlayerData *cd)
+cd_try_open(CDPlayerData *cd, int *err)
 {
-    int err;
     if(cd->cdrom_device == NULL) {
-        cd->cdrom_device = cdrom_open(cd->devpath, &err);
+        cd->cdrom_device = cdrom_open(cd->devpath, err);
         return cd->cdrom_device != NULL;
     }
     return TRUE;
@@ -745,8 +771,9 @@ cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd)
 {
     cdrom_device_status_t stat;
     gboolean description = FALSE;
+    int err;
 
-    if (cd_try_open(cd)) {
+    if (cd_try_open(cd, &err)) {
         if (cdrom_get_status(cd->cdrom_device, &stat) == DISC_NO_ERROR)
         {
             switch (stat.audio_status)
@@ -821,9 +848,13 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
     cdrom_device_status_t stat;
     int status;
     int ret;
+    int err;
 
-    if(!cd_try_open(cd))
+    if(!cd_try_open(cd, &err)) {
+        if (err == EACCES) 
+            show_error (cd);
         return;
+    }
 
     status = cdrom_get_status(cd->cdrom_device, &stat);
     if (status == DISC_NO_ERROR) {
@@ -857,7 +888,7 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						 _("Audio device is busy, or being used by another application"),
 						 NULL);
 		gtk_window_set_screen (GTK_WINDOW (dialog),
@@ -874,8 +905,13 @@ static void
 cdplayer_stop(GtkWidget * w, gpointer data)
 {
     CDPlayerData *cd = data;
-    if(!cd_try_open(cd))
+    int err;
+    
+    if(!cd_try_open(cd, &err)) {
+        if (err == EACCES) 
+            show_error (cd);
         return;
+    }
     cdrom_stop(cd->cdrom_device);
 
 }
@@ -884,8 +920,12 @@ static void
 cdplayer_prev(GtkWidget * w, gpointer data)
 {
     CDPlayerData *cd = data;
-    if(!cd_try_open(cd))
+    int err;
+    if(!cd_try_open(cd, &err)) {
+        if (err == EACCES)
+            show_error (cd);
         return;
+    }
     cdrom_prev(cd->cdrom_device);
 
 }
@@ -894,8 +934,12 @@ static void
 cdplayer_next(GtkWidget * w, gpointer data)
 {
     CDPlayerData *cd = data;
-    if(!cd_try_open(cd))
+    int err;
+    if(!cd_try_open(cd, &err)) {
+        if (err == EACCES)
+            show_error (cd);
         return;
+    }
     cdrom_next(cd->cdrom_device);
 
 }
@@ -905,8 +949,12 @@ cdplayer_eject(GtkWidget * w, gpointer data)
 {
     cdrom_device_status_t stat;
     CDPlayerData *cd = data;
-    if(!cd_try_open(cd))
+    int err;
+    if(!cd_try_open(cd, &err)) {
+        if (err == EACCES)
+            show_error (cd);
         return;
+     }
     if(cdrom_get_status(cd->cdrom_device, &stat) == DISC_TRAY_OPEN)
         /*
           FIXME: if there is no disc, we get TRAY_OPEN even
