@@ -24,6 +24,7 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <panel-applet.h>
+#include <egg-screen-exec.h>
 
 #include "drivemount.h"
 #include "properties.h"
@@ -65,24 +66,34 @@ static void dnd_drag_begin_cb (GtkWidget *widget, GdkDragContext *context,
 static void dnd_set_data_cb (GtkWidget *widget, GdkDragContext *context,
 			     GtkSelectionData *selection_data, guint info,
 			     guint time, gpointer data);
-static gboolean applet_button_press_cb (GtkWidget *widget,
-					GdkEventButton *event, gpointer data);
 static void applet_change_orient (GtkWidget *w, PanelAppletOrient o,
 				  gpointer data);
 static void applet_change_pixel_size (GtkWidget *w, int size, gpointer data);
+static gboolean button_press_hack (GtkWidget      *widget,
+				   GdkEventButton *event,
+				   GtkWidget      *applet);
 static void destroy_drive_widget (GtkWidget *widget, gpointer data);
-static void browse_cb (PanelApplet *widget, gpointer data);
-static void eject_cb (PanelApplet *applet, gpointer data);
-static void help_cb (PanelApplet *widget, gpointer data);
-static void about_cb (PanelApplet *widget, gpointer data);
 static gint device_is_in_mountlist (DriveData *dd);
 static gint get_device (const gchar *file);
 static gint device_is_mounted (DriveData *dd);
 static void update_pixmap (DriveData *dd, gint t);
 static gint drive_update_cb (gpointer data);
-static gint mount_cb (GtkWidget *widget, gpointer data);
+static void mount_cb (GtkWidget *widget, DriveData *dd);
 static void eject (DriveData *dd);
 
+static void browse_cb (BonoboUIComponent *uic,
+		       DriveData         *drivemount,
+		       const char        *verb);
+static void eject_cb  (BonoboUIComponent *uic,
+		       DriveData         *drivemount,
+		       const char        *verb);
+static void help_cb   (BonoboUIComponent *uic,
+		       DriveData         *drivemount,
+		       const char        *verb);
+static void about_cb  (BonoboUIComponent *uic,
+		       DriveData         *drivemount,
+		       const char        *verb);
+
 /*
  *-------------------------------------------------------------------------
  *icon struct
@@ -138,7 +149,7 @@ static IconData icon_list[] = {
 static gint icon_list_count = 6;
 
 /* Bonobo Verbs for our popup menu */
-static const BonoboUIVerb applet_menu_verbs[] = {
+static const BonoboUIVerb applet_menu_verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("Browse", browse_cb),
 	BONOBO_UI_UNSAFE_VERB ("Eject", eject_cb),
 	BONOBO_UI_UNSAFE_VERB ("Properties", properties_show),
@@ -195,7 +206,7 @@ applet_fill (PanelApplet *applet)
 	dd->orient = panel_applet_get_orient (PANEL_APPLET (applet));
 	dd->sizehint = panel_applet_get_size (PANEL_APPLET (applet));
 	g_signal_connect (G_OBJECT (dd->button), "button_press_event",
-			  G_CALLBACK (applet_button_press_cb), applet);
+			  G_CALLBACK (button_press_hack), applet);
 
 	g_signal_connect (G_OBJECT (applet), "change_orient",
 			  G_CALLBACK (applet_change_orient), dd);
@@ -244,19 +255,21 @@ create_drive_widget (PanelApplet *applet)
 	return dd;
 }
 
+/* This is a hack around the fact that gtk+ doesn't
+ * propogate button presses on button2/3.
+ */
 static gboolean
-applet_button_press_cb (GtkWidget *widget, GdkEventButton *event,
-			gpointer data)
+button_press_hack (GtkWidget      *widget,
+                   GdkEventButton *event,
+                   GtkWidget      *applet)
 {
-	PanelApplet *applet = PANEL_APPLET (data);
+    if (event->button == 3 || event->button == 2) {
+	gtk_propagate_event (applet, (GdkEvent *) event);
 
-	if (event->button == 1) {
-		return FALSE;
-	} else {
-		GTK_WIDGET_CLASS (PANEL_APPLET_GET_CLASS (applet))->
-			button_press_event (data, event);
-	}
 	return TRUE;
+    }
+
+    return FALSE;
 }
 
 static void
@@ -359,50 +372,105 @@ destroy_drive_widget (GtkWidget *widget, gpointer data)
 }
 
 static void
-browse_cb (PanelApplet *widget, gpointer data)
+browse_cb (BonoboUIComponent *uic,
+	   DriveData         *drivemount,
+	   const char        *verb)
 {
-	DriveData *dd = data;
-	char *str;
-	char *argv[4];
+	GError *error = NULL;
+	char   *command;
 
-	/* attempt to mount first, otherwise, what is the point? */
-	if (!dd->mounted) {
-		mount_cb (NULL, dd);
-		if (!dd->mounted)
-			return;	/* failed to mount, so abort */
-	}
+	if (!drivemount->mounted)
+		mount_cb (NULL, drivemount);
 
-	/* Do we have nautilus */
-	str = gnome_is_program_in_path ("nautilus");
-	if (str != NULL) {
-		argv[0] = "nautilus";
-		argv[1] = dd->mount_point;
-		argv[2] = NULL;
-		gnome_execute_async (NULL, 2, argv);
-	} else {
-		argv[0] = "gmc-client";
-		argv[1] = "--create-window";
-		argv[2] = dd->mount_point;
-		argv[3] = NULL;
-		gnome_execute_async (NULL, 3, argv);
+	if (!drivemount->mounted)
+		return;
+
+	command = g_strdup_printf ("nautilus %s", drivemount->mount_point);
+	egg_screen_execute_command_line_async (
+		gtk_widget_get_screen (drivemount->applet), command, &error);
+	g_free (command);
+	if (error) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("There was an error executing '%s': %s"),
+						 command,
+						 error->message);
+
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (drivemount->applet));
+
+		gtk_widget_show (dialog);
+
+		g_error_free (error);
 	}
 }
 
 static void
-eject_cb (PanelApplet *applet, gpointer data)
+eject_cb (BonoboUIComponent *uic,
+	  DriveData         *drivemount,
+	  const char        *verb)
 {
-	DriveData *dd = data;
-
-	eject (dd);
+	eject (drivemount);
 }
 
 static void
-help_cb (PanelApplet *widget, gpointer data)
+help_cb (BonoboUIComponent *uic,
+	 DriveData         *drivemount,
+	 const char        *verb)
+
 {
+	GError *error = NULL;
+	static GnomeProgram *applet_program = NULL;
+	
+	if (!applet_program) {
+		int argc = 1;
+		char *argv[2] = { "drivemount" };
+		applet_program = gnome_program_init ("drivemount", VERSION,
+						      LIBGNOME_MODULE, argc, argv,
+     						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
+	}
+
+	egg_help_display_desktop_on_screen (
+			applet_program, "drivemount", "drivemount", "drivemount",
+			gtk_widget_get_screen (GTK_WIDGET (uic)),
+			&error);
+
+	if (error) {
+		GtkWidget *error_dialog;
+
+		error_dialog = gtk_message_dialog_new (
+				NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				_("There was an error displaying help: %s"),
+				error->message);
+
+		g_signal_connect (error_dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (error_dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (error_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (uic)));
+		gtk_widget_show (error_dialog);
+		g_error_free (error);
+	}
 }
 
 static void
-about_cb (PanelApplet *widget, gpointer data)
+about_cb (BonoboUIComponent *uic,
+	  DriveData         *drivemount,
+	  const char        *verb)
 {
 	static GtkWidget *about = NULL;
    	GdkPixbuf        *pixbuf;
@@ -421,9 +489,10 @@ about_cb (PanelApplet *widget, gpointer data)
 
 	const gchar *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gdk_window_show (about->window);
-		gdk_window_raise (about->window);
+	if (about) {
+		gtk_window_set_screen (GTK_WINDOW (about),
+				       gtk_widget_get_screen (drivemount->applet));
+		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
 
@@ -448,6 +517,8 @@ about_cb (PanelApplet *widget, gpointer data)
    		gdk_pixbuf_unref (pixbuf);
    
    	gtk_window_set_wmclass (GTK_WINDOW (about), "disk mounter", "Disk Mounter");
+	gtk_window_set_screen (GTK_WINDOW (about),
+			       gtk_widget_get_screen (drivemount->applet));
    	g_signal_connect (G_OBJECT (about), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &about);
 	gtk_widget_show (about);
@@ -540,6 +611,7 @@ update_pixmap (DriveData *dd, gint t)
 	GdkPixbuf *scaled;
 	char **pmap_d_in;
 	char **pmap_d_out;
+	gint pixmap;
 
 	gint width;
 	gint height;
@@ -549,11 +621,12 @@ update_pixmap (DriveData *dd, gint t)
 
 	gint hint = dd->sizehint;
 
+	pixmap = dd->device_pixmap;
 	if (dd->device_pixmap > icon_list_count - 1)
-		dd->device_pixmap = 0;
+		pixmap = 0;
 	if (dd->device_pixmap < 0
 	    && (!dd->custom_icon_in || !dd->custom_icon_out))
-		dd->device_pixmap = 0;
+		pixmap = 0;
 
 	if (!dd->button_pixmap) {
 		dd->button_pixmap = gtk_image_new ();
@@ -562,7 +635,7 @@ update_pixmap (DriveData *dd, gint t)
 		gtk_widget_show (dd->button_pixmap);
 	}
 
-	if (dd->device_pixmap < 0) {
+	if (pixmap < 0) {
 		if (t) {
 			pixbuf = gdk_pixbuf_new_from_file (dd->custom_icon_in,
 							   NULL);
@@ -607,8 +680,8 @@ update_pixmap (DriveData *dd, gint t)
 
 		if (dd->orient == PANEL_APPLET_ORIENT_LEFT
 		    || dd->orient == PANEL_APPLET_ORIENT_RIGHT || hint <= 36) {
-			pmap_d_in = icon_list[dd->device_pixmap].pmap_h_in;
-			pmap_d_out = icon_list[dd->device_pixmap].pmap_h_out;
+			pmap_d_in = icon_list[pixmap].pmap_h_in;
+			pmap_d_out = icon_list[pixmap].pmap_h_out;
 		} else {
 			gint tmp;
 
@@ -616,8 +689,8 @@ update_pixmap (DriveData *dd, gint t)
 			width = height;
 			height = tmp;
 
-			pmap_d_in = icon_list[dd->device_pixmap].pmap_v_in;
-			pmap_d_out = icon_list[dd->device_pixmap].pmap_v_out;
+			pmap_d_in = icon_list[pixmap].pmap_v_in;
+			pmap_d_out = icon_list[pixmap].pmap_v_out;
 		}
 
 		if (t)
@@ -689,10 +762,10 @@ drive_update_cb (gpointer data)
  *-------------------------------------------------------------------------
  */
 
-static gint
-mount_cb (GtkWidget *widget, gpointer data)
+static void
+mount_cb (GtkWidget *widget,
+	  DriveData *dd)
 {
-	DriveData *dd = data;
 	gchar *command_line;
 	gchar buf[512];
 	FILE *fp;
@@ -704,9 +777,10 @@ mount_cb (GtkWidget *widget, gpointer data)
 
 	/* Stop the user from displaying zillions of error messages */
 	if (dd->error_dialog) {
-		gdk_window_show (dd->error_dialog->window);
-		gdk_window_raise (dd->error_dialog->window);
-		return FALSE;
+		gtk_window_set_screen (GTK_WINDOW (dd->error_dialog),
+				       gtk_widget_get_screen (dd->applet));
+		gtk_window_present (GTK_WINDOW (dd->error_dialog));
+		return;
 	}
 
 	if (!check) {
@@ -722,17 +796,22 @@ mount_cb (GtkWidget *widget, gpointer data)
 	if (!fp) {
 		printf ("unable to run command: %s\n", command_line);
 		g_free (command_line);
-		return FALSE;
+		return;
 	}
 
 	str = g_string_new (NULL);
 
 	while (fgets (buf, sizeof (buf), fp) != NULL) {
 		gchar *b = buf;
-
+		const gchar *charset;
+		if (!g_get_charset (&charset)) {
+			b = g_convert (buf, -1, "UTF-8", charset, NULL, NULL, NULL);
+		} else {
+			b =g_strdup (buf);
+		}
 		g_string_append (str, b);
+		g_free (b);
 	}
-
 	pclose (fp);
 
 	dd->mounted = device_is_mounted (dd);
@@ -752,6 +831,8 @@ mount_cb (GtkWidget *widget, gpointer data)
 						     NULL, GTK_DIALOG_MODAL,
 						     GTK_STOCK_OK,
 						     GTK_RESPONSE_OK, NULL);
+		gtk_window_set_screen (GTK_WINDOW (dd->error_dialog),
+				       gtk_widget_get_screen (dd->applet));
 		hbox = gtk_hbox_new (FALSE, 0);
 		gtk_box_pack_start (GTK_BOX
 				    (GTK_DIALOG (dd->error_dialog)->vbox), hbox,
@@ -771,7 +852,7 @@ mount_cb (GtkWidget *widget, gpointer data)
 	}
 	g_string_free (str, TRUE);
 	g_free (command_line);
-	return FALSE;
+	return;
 }
 
 static void
