@@ -33,6 +33,7 @@ gchar *theme_directories[] = {
         "~/.gnome/geyes-themes/"
 };
 #define NUM_THEME_DIRECTORIES 2
+#define HIG_IDENTATION  "    "
 
 static void
 parse_theme_file (EyesApplet *eyes_applet, FILE *theme_file)
@@ -88,7 +89,11 @@ load_theme (EyesApplet *eyes_applet, const gchar *theme_dir)
         file_name = g_strdup_printf("%s%s",theme_dir,"/config");
         theme_file = fopen (file_name, "r");
         if (theme_file == NULL) {
-                g_error ("Unable to open theme file.");
+        	g_free (eyes_applet->theme_dir);
+        	eyes_applet->theme_dir = g_strdup_printf (GEYES_THEMES_DIR "Default-tiny/");
+        	g_free (file_name);
+                file_name = g_strdup (GEYES_THEMES_DIR "Default-tiny/config");
+                theme_file = fopen (file_name, "r");
         }
         
         parse_theme_file (eyes_applet, theme_file);
@@ -201,6 +206,9 @@ properties_cb (BonoboUIComponent *uic,
 	       const gchar       *verbname)
 {
 	GtkWidget *pbox, *hbox;
+	GtkWidget *vbox, *indent;
+	GtkWidget *categories_vbox;
+	GtkWidget *category_vbox, *control_vbox;
         GtkWidget *tree;
         GtkWidget *label;
         GtkListStore *model;
@@ -211,7 +219,12 @@ properties_cb (BonoboUIComponent *uic,
         DIR *dfd;
         struct dirent *dp;
         int i;
+#ifdef PATH_MAX
         gchar filename [PATH_MAX];
+#else
+	gchar *filename;
+#endif
+        gchar *title;
      
 	if (eyes_applet->prop_box.pbox) {
 		gtk_window_set_screen (
@@ -228,19 +241,55 @@ properties_cb (BonoboUIComponent *uic,
 					     NULL);
 	gtk_window_set_screen (GTK_WINDOW (pbox),
 			       gtk_widget_get_screen (GTK_WIDGET (eyes_applet->applet)));
+        gtk_window_set_resizable (GTK_WINDOW (pbox), FALSE);
         gtk_dialog_set_default_response(GTK_DIALOG (pbox), GTK_RESPONSE_CLOSE);
+        gtk_dialog_set_has_separator (GTK_DIALOG (pbox), FALSE);
+        gtk_container_set_border_width (GTK_CONTAINER (pbox), 5);
 
         g_signal_connect (pbox, "response",
 			  G_CALLBACK (presponse_cb),
 			  eyes_applet);
 	
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pbox)->vbox), hbox, FALSE, FALSE, 2);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	gtk_widget_show (vbox);
 	
-	label = gtk_label_new_with_mnemonic (_("_Theme Name:"));
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pbox)->vbox), vbox,
+			    TRUE, TRUE, 0);
 
+	categories_vbox = gtk_vbox_new (FALSE, 18);
+	gtk_box_pack_start (GTK_BOX (vbox), categories_vbox, TRUE, TRUE, 0);
+	gtk_widget_show (categories_vbox);
+
+	category_vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (categories_vbox), category_vbox, TRUE, TRUE, 0);
+	gtk_widget_show (category_vbox);
+	
+	title = g_strconcat ("<span weight=\"bold\">", _("Themes"), "</span>", NULL);
+	label = gtk_label_new (_(title));
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_box_pack_start (GTK_BOX (category_vbox), label, FALSE, FALSE, 0);
+	g_free (title);
+	
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (category_vbox), hbox, TRUE, TRUE, 0);
+	gtk_widget_show (hbox);
+	
+	indent = gtk_label_new (HIG_IDENTATION);
+	gtk_label_set_justify (GTK_LABEL (indent), GTK_JUSTIFY_LEFT);
+	gtk_box_pack_start (GTK_BOX (hbox), indent, FALSE, FALSE, 0);
+	gtk_widget_show (indent);
+	
+	control_vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (hbox), control_vbox, TRUE, TRUE, 0);
+	gtk_widget_show (control_vbox);
+	
+	label = gtk_label_new_with_mnemonic (_("_Select a theme:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_box_pack_start (GTK_BOX (control_vbox), label, FALSE, FALSE, 0);
+	
 	model = gtk_list_store_new (1, G_TYPE_STRING);
 	tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
@@ -264,9 +313,13 @@ properties_cb (BonoboUIComponent *uic,
                                         gchar *elems[2] = {NULL, NULL };
                                         gchar *theme_dir;
 					elems[0] = filename;
+#ifdef PATH_MAX
                                         strcpy (filename, 
                                                 theme_directories[i]);
                                         strcat (filename, dp->d_name);
+#else
+					asprintf (&filename, theme_directories[i], dp->d_name);
+#endif
                                         gtk_list_store_insert (model, &iter, 0);
                                         gtk_list_store_set (model, &iter, 0, &filename, -1);
                                         theme_dir = g_strdup_printf ("%s/", filename);
@@ -286,8 +339,11 @@ properties_cb (BonoboUIComponent *uic,
                         closedir (dfd);
                 }
         }
+#ifndef PATH_MAX
+	g_free (filename);
+#endif
         
-        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pbox)->vbox), tree, TRUE, TRUE, 2);
+        gtk_box_pack_start (GTK_BOX (control_vbox), tree, TRUE, TRUE, 0);
         
         gtk_widget_show_all (pbox);
         
