@@ -114,8 +114,6 @@ gkb_draw (GKB * gkb)
 
 }
 
-
-#define GKB_SMALL_PANEL_SIZE 25	/* less than */
 /**
  * count_sizes:
  * @gkb:
@@ -803,6 +801,10 @@ static const BonoboUIVerb gkb_menu_verbs [] = {
 static void gkb_destroy (GtkWidget * widget, gpointer data)
 {
 	GKB *gkb = (GKB *)data;
+	
+	if (gkb->propwindow)
+		gtk_widget_destroy (gkb->propwindow);
+	gkb_update_handlers (gkb, TRUE);
 	g_free(gkb);
 }
 
