@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.55 2003/03/07 00:32:23 markmc Exp $ */
+/* $Id: gweather-applet.c,v 1.58 2003/06/01 01:34:54 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -31,6 +31,17 @@
 #include "gweather-applet.h"
 #include "location.h"
 
+
+static void
+set_all_tooltips (GWeatherApplet *applet, gchar *text)
+{
+	gint i;
+
+	for (i=0; i<=applet->gweather_info->numforecasts-1; i++) {
+		gtk_tooltips_set_tip (applet->tooltips,applet->events[i], text, NULL);
+	}
+}
+
 void update_display (GWeatherApplet *applet)
 {
 	WeatherInfo *info = applet->gweather_info;
@@ -52,12 +63,11 @@ void update_display (GWeatherApplet *applet)
 	tmp = g_strdup_printf ("%d\302\260", info->curtemp);
 	gtk_label_set_text (GTK_LABEL (applet->labels[0]), tmp);
 	g_free (tmp);
-	
-g_print ("update \n");
+
 	list = info->forecasts;
-	for (i=1; i<=applet->gweather_info->numforecasts; i++) {
+	for (i=1; i<=applet->gweather_info->numforecasts-1; i++) {
 		WeatherForecast *forecast = NULL;
-		gint avgtemp;
+		
 		if (!list)
 			continue;
 		forecast = list->data;	
@@ -74,15 +84,20 @@ g_print ("update \n");
 			gtk_tooltips_set_tip (applet->tooltips,applet->events[i], NULL, NULL);
 		pixbuf = get_conditions_pixbuf (forecast->wid);
 		gtk_image_set_from_pixbuf (GTK_IMAGE (applet->images[i]), pixbuf);
-		avgtemp = 0.5 * (forecast->low + forecast->high);
-		tmp = g_strdup_printf ("%d\302\260", avgtemp);
+		tmp = g_strdup_printf ("%d/%d\302\260", forecast->high, forecast->low);
 		gtk_label_set_text (GTK_LABEL (applet->labels[i]), tmp);
 		g_free (tmp);
 		list = g_list_next (list);
 	
 	}
 
-g_print ("done updating display \n");
+	if (!info->success) {
+		set_all_tooltips (applet, _("There was an error retrieving the forecast"));
+		for (i=0; i<=applet->gweather_info->numforecasts-1; i++) {
+			gtk_image_set_from_pixbuf (GTK_IMAGE (applet->images[i]), 
+							  get_conditions_pixbuf (-1));
+		}
+	}
 }
 
 void place_widgets (GWeatherApplet *gw_applet)
@@ -96,16 +111,16 @@ void place_widgets (GWeatherApplet *gw_applet)
     if ((gw_applet->orient == PANEL_APPLET_ORIENT_LEFT) || 
          (gw_applet->orient == PANEL_APPLET_ORIENT_RIGHT)) {
 	 horiz = FALSE;
-         gw_applet->box = gtk_vbox_new (FALSE, 2);         
+         gw_applet->box = gtk_vbox_new (FALSE, 4);         
     }
     else {
-         gw_applet->box = gtk_hbox_new (FALSE, 2);
+         gw_applet->box = gtk_hbox_new (FALSE, 4);
 	 horiz = TRUE;
     }
     
     gtk_container_add (GTK_CONTAINER (gw_applet->applet), gw_applet->box);
 
-    for (i=0; i<=gw_applet->gweather_info->numforecasts; i++) {
+    for (i=0; i<=gw_applet->gweather_info->numforecasts-1; i++) {
 		gchar *tmp;
 		gw_applet->events[i] = gtk_event_box_new ();
 		gtk_box_pack_start (GTK_BOX (gw_applet->box), 
@@ -122,7 +137,10 @@ void place_widgets (GWeatherApplet *gw_applet)
 		gw_applet->labels[i] = gtk_label_new (NULL);
 		gtk_box_pack_start (GTK_BOX (gw_applet->boxes[i]), 
 				                      gw_applet->labels[i], FALSE, FALSE, 0);
-		tmp = g_strdup_printf ("%d\302\260", 0);
+		if (i ==0)
+			tmp = g_strdup_printf ("%d\302\260", 0);
+		else
+			tmp = g_strdup_printf ("%d/%d\302\260", 0, 0);
 		gtk_label_set_text (GTK_LABEL (gw_applet->labels[i]), tmp);
 		g_free (tmp);
     }
@@ -130,7 +148,7 @@ void place_widgets (GWeatherApplet *gw_applet)
     gtk_widget_show_all (GTK_WIDGET (gw_applet->applet));
 
     if (!gw_applet->gweather_pref.show_labels) {
-	for (i=0; i<=gw_applet->gweather_info->numforecasts; i++) {
+	for (i=0; i<=gw_applet->gweather_info->numforecasts-1; i++) {
 		gtk_widget_hide (gw_applet->labels[i]);
 	}
     }
@@ -289,13 +307,8 @@ void gweather_update (GWeatherApplet *gw_applet)
     gboolean update_success;
     gint i;
 
-    gtk_tooltips_set_tip(gw_applet->tooltips, GTK_WIDGET(gw_applet->applet), 
-    			                 _("Updating..."), NULL);
+    set_all_tooltips (gw_applet, _("Updating..."));
     
-    for (i=0; i<=gw_applet->gweather_info->numforecasts; i++) {
-    	gtk_tooltips_set_tip (gw_applet->tooltips, gw_applet->events[i], NULL, NULL);
-    }
-
     update_success = update_weather (gw_applet);
     
     return;
