@@ -23,6 +23,8 @@
 #include <panel-applet-gconf.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <stdlib.h>
+#include <ctype.h>
 
 #include "popcheck.h"
 #include "remote-helper.h"
@@ -30,6 +32,15 @@
 #include "egg-screen-help.h"
 #include "egg-screen-exec.h"
 
+#define ENCODE 1  
+#define DECODE 0
+
+static const char *to_b64 =
+ "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+/* encode 72 characters per line */
+#define CHARS_PER_LINE 72 
+
 typedef enum {
 	MAILBOX_LOCAL,
 	MAILBOX_LOCALDIR,
@@ -37,6 +48,12 @@ typedef enum {
 	MAILBOX_IMAP
 } MailboxType;
 
+typedef struct
+{
+        int     size;
+        char    *data;
+} memchunk;
+
 typedef struct _MailCheck MailCheck;
 struct _MailCheck {
 	char *mail_file;
@@ -57,8 +74,9 @@ struct _MailCheck {
 	/* interval to check for mails in milliseconds */
 	guint update_freq;
 
-	/* whether to set mc->newmail and mc->unreadmail to 0 if the applet was clicked */
+	/* Pref is reset_on_clicked. Show animation is intended to restart animation for newmail */
 	gboolean reset_on_clicked;
+	gboolean show_animation;
 
 	/* execute a command when the applet is clicked (launch email prog) */
 	char *clicked_cmd;
@@ -134,14 +152,19 @@ struct _MailCheck {
 	GtkWidget *remote_server_entry, *remote_username_entry, *remote_password_entry, *remote_folder_entry;
 	GtkWidget *remote_server_label, *remote_username_label, *remote_password_label, *remote_folder_label;
 	GtkWidget *remote_option_menu;
+	GtkWidget *remote_password_checkbox;
 	GtkWidget *play_sound_check;
         
 	char *remote_server, *remote_username, *remote_password, *real_password, *remote_folder;
+	char *remote_encrypted_password;
+
 	MailboxType mailbox_type; /* local = 0; maildir = 1; pop3 = 2; imap = 3 */
         MailboxType mailbox_type_temp;
 
 	gboolean play_sound;
 
+	gboolean save_remote_password;
+
 	int type; /*mailcheck = 0; mailbox = 1 */
 	
 	off_t oldsize;
@@ -154,6 +177,15 @@ struct _MailCheck {
 	gpointer remote_handle;
 };
 
+static char*           b64enc (const memchunk *chunk);
+static memchunk*       b64dec (const char *string);
+static memchunk*       memchunkAlloc (int size);
+static char*           util_base64 (char *data, int flag);
+static void            remote_password_save_toggled (GtkToggleButton *button, 
+				gpointer data);
+static void            remote_password_changed (GtkEntry *entry, gpointer data);
+static gboolean        focus_out_cb (GtkWidget *entry, GdkEventFocus *event, gpointer data);
+
 static int mail_check_timeout (gpointer data);
 static void after_mail_check (MailCheck *mc);
 
@@ -169,6 +201,280 @@ static void null_remote_handle (gpointer data);
 #define WANT_BITMAPS(x) (x == REPORT_MAIL_USE_ANIMATION || x == REPORT_MAIL_USE_BITMAP)
 #define HIG_IDENTATION  "    "
 
+static memchunk* 
+memchunkAlloc(int size)
+{
+	memchunk *tmp = (memchunk*) calloc(1, sizeof(memchunk));
+
+	if (tmp) {
+		tmp->size = size;
+		tmp->data = (char *) malloc(size);
+
+		if (tmp->data == (char *) 0) {
+			free(tmp);
+			tmp = 0;
+		}
+	}
+
+	return tmp;
+}
+
+/*
+ * b64enc:
+ * @chunk: A structure which contains the data to be encrypted.
+ * 
+ * This routine encrypts the data using base64 encryption technique
+ *
+ * Returns the encrypted data in a character array.
+ */
+
+
+static char* 
+b64enc(const memchunk *chunk)
+{
+	int div = chunk->size / 3;
+	int rem = chunk->size % 3;
+	int chars = div*4 + rem + 1;
+	int newlines = (chars + CHARS_PER_LINE - 1) / CHARS_PER_LINE;
+
+	const char *data = chunk->data;
+	char *string = (char *) malloc(chars + newlines + 1);
+
+	if (string) {
+		register char* buf = string;
+
+		chars = 0;
+
+		/*@+charindex@*/
+		while (div > 0) {
+			buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
+			buf[1] = to_b64[((data[0] << 4) & 0x30) +
+				 ((data[1] >> 4) & 0xf)];
+			buf[2] = to_b64[((data[1] << 2) & 0x3c) +
+				 ((data[2] >> 6) & 0x3)];
+			buf[3] = to_b64[  data[2] & 0x3f];
+			data += 3;
+			buf += 4;
+			div--;
+			chars += 4;
+			if (chars == CHARS_PER_LINE) {
+				chars = 0;
+				*(buf++) = '\n';
+			}
+		}
+
+		switch (rem) {
+			case 2:
+				buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
+				buf[1] = to_b64[((data[0] << 4) & 0x30) +
+					 ((data[1] >> 4) & 0xf)];
+				buf[2] = to_b64[ (data[1] << 2) & 0x3c];
+				buf[3] = '=';
+				buf += 4;
+				chars += 4;
+				break;
+			case 1:
+				buf[0] = to_b64[ (data[0] >> 2) & 0x3f];
+				buf[1] = to_b64[ (data[0] << 4) & 0x30];
+				buf[2] = '=';
+				buf[3] = '=';
+				buf += 4;
+				chars += 4;
+				break;
+		}
+		/*@=charindex@*/
+
+	 /*      *(buf++) = '\n'; This would result in a buffer overrun */
+		*buf = '\0';
+	 }
+
+	return string;
+}
+
+/*
+ * b64dec:
+ * @string: A character array which contains the data to be decoded.
+ *
+ * This routine decrypts the data using base64 decryption technique
+ *
+ * Returns the decrypted data.
+ */
+
+static memchunk* 
+b64dec(const char *string)
+{
+
+/* return a decoded memchunk, or a null pointer in case of failure */
+
+	memchunk *rc = 0;
+
+	if (string) {
+	register int length = strlen(string);
+
+	/* do a format verification first */
+		if (length > 0) {
+			register int count = 0, rem = 0;
+			register const char* tmp = string;
+
+			while (length > 0) {
+				register int skip = strspn(tmp, to_b64);
+				count += skip;
+				length -= skip;
+				tmp += skip;
+				if (length > 0) {
+					register int i, vrfy =
+						 strcspn(tmp, to_b64);
+
+					for (i = 0; i < vrfy; i++) {
+						if (isspace(tmp[i]))
+							continue;
+
+						if (tmp[i] == '=') {
+							/* we should check if
+							 * we're close to the
+							 * end of the string */
+							rem = count % 4;
+
+							/* rem must be either
+							 * 2 or 3, otherwise
+							 * no '=' should be
+							 * here */
+							if (rem < 2)
+								return 0;
+
+							/* end-of-message
+							 * recognized */
+							break;
+						} else {
+							/* Transmission error */
+
+							return 0;
+						}
+					}
+
+					length -= vrfy;
+					tmp += vrfy;
+				}
+			}
+
+			rc = memchunkAlloc((count / 4) * 3 + (rem ? (rem - 1) : 0));
+
+			if (rc) {
+				if (count > 0) {
+					register int i, qw = 0, tw = 0;
+					register char * data = rc->data;
+
+					length = strlen(tmp = string);
+
+					for (i = 0; i < length; i++) {
+						register char ch = string[i];
+						register char bits;
+
+						if (isspace(ch))
+							continue;
+
+						bits = 0;
+						if ((ch >= 'A') && (ch <= 'Z')) {
+							bits = (char)
+							       (ch - 'A');
+						}
+						else if ((ch >= 'a') &&
+							 (ch <= 'z')) {
+							bits = (char)
+							       (ch - 'a' + 26);
+						}
+						else if ((ch >= '0') &&
+							 (ch <= '9')) {
+							bits = (char)
+							       (ch - '0' + 52);
+						}
+						else if (ch == '=')
+							break;
+
+						switch (qw++) {
+							case 0:
+								data[tw+0] =
+									 (bits << 2) & 0xfc;
+								break;
+							case 1:
+								data[tw+0] |=
+									 (bits >> 4) & 0x03;
+								data[tw+1] =
+									 (bits << 4) & 0xf0;
+								break;
+							case 2:
+								data[tw+1] |=
+									 (bits >> 2) & 0x0f;
+								data[tw+2] =
+									 (bits << 6) & 0xc0;
+								break;
+							case 3:
+								data[tw+2] |=
+									 bits & 0x3f;
+								break;
+						}
+
+						if (qw == 4) {
+							qw = 0;
+							tw += 3;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	return rc;
+}
+
+
+/*
+ * util_base64:
+ * @data: A character array which contains the data.
+ * @flag: Indicates the action to be performed ENCODE/DECODE
+ *
+ * This is the interface called by other routines for encoding or decoding using
+ * base64 encryption/decryption technique. The action is performed based on the flag.
+ *
+ * Returns either the encrypted data or the decrypted data based on the flag.
+ */
+
+static char* 
+util_base64(char* data, int flag)
+{
+	char *enc_data;
+	memchunk chunk;
+	memchunk *ret_data;
+	char *dec_data;
+
+	switch (flag) {
+		case ENCODE :
+
+			chunk.data = data;
+			chunk.size = strlen(data);
+			enc_data = b64enc(&chunk);
+
+			return enc_data;
+
+		case DECODE :
+
+			ret_data = (memchunk *) b64dec((char *)data);
+			if(ret_data) {
+				dec_data = g_strndup(ret_data->data,
+						     ret_data->size);
+				free (ret_data);
+			}
+			else
+				dec_data = NULL;
+
+			return dec_data;
+
+		default :
+			return NULL;
+
+	}
+}
+
 static void
 set_tooltip (GtkWidget  *applet,
 	     const char *tip)
@@ -202,7 +508,7 @@ mailcheck_execute_shell (MailCheck  *mailcheck,
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						 _("There was an error executing %s: %s"),
 						 command,
 						 error->message);
@@ -280,15 +586,15 @@ check_remote_mailbox (MailCheck *mc)
 						       mc->remote_username,
 						       mc->real_password);
 	else if (mc->mailbox_type == MAILBOX_IMAP)
-		helper_imap_check (got_remote_answer,
-			           error_handler,
-				   mc,
-				   null_remote_handle,
-				   NULL,
-				   mc->remote_server,
-				   mc->remote_username,
-				   mc->real_password,
-				   mc->remote_folder);
+		mc->remote_handle = helper_imap_check (got_remote_answer,
+						       error_handler,
+						       mc,
+						       null_remote_handle,
+						       NULL,
+						       mc->remote_server,
+						       mc->remote_username,
+						       mc->real_password,
+						       mc->remote_folder);
 }
 
 static void
@@ -317,6 +623,7 @@ get_remote_password (MailCheck *mc)
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *entry;
+	GtkWidget *save_password_checkbox;
 
 	if (mc->password_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (mc->password_dialog),
@@ -353,6 +660,29 @@ get_remote_password (MailCheck *mc)
 	gtk_widget_show_all (hbox);
 	gtk_widget_grab_focus (GTK_WIDGET (entry));
 
+	g_signal_connect (G_OBJECT(entry), "focus_out_event",
+			 G_CALLBACK(focus_out_cb), mc);
+
+	g_signal_connect (G_OBJECT(entry), "activate",
+			  G_CALLBACK(remote_password_changed), mc);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, 
+			    FALSE, FALSE, GNOME_PAD_SMALL);
+
+	gtk_widget_show (hbox);  
+	
+	save_password_checkbox = gtk_check_button_new_with_mnemonic (
+						_("_Save password to disk"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (save_password_checkbox), 
+				      mc->save_remote_password);
+
+	gtk_widget_show (save_password_checkbox);
+	gtk_box_pack_start (GTK_BOX (hbox), save_password_checkbox, FALSE, FALSE, 0);
+	
+	g_signal_connect (G_OBJECT (save_password_checkbox), "toggled", 
+			  G_CALLBACK(remote_password_save_toggled), mc);
+
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 
@@ -387,50 +717,34 @@ static void
 error_handler (int error, gpointer data)
 {
 	MailCheck *mc = data;
-	GtkWidget *dialog;
 	gchar *details;
-
-	/* Disable automatic updating */
-	mc->auto_update = FALSE;
-
-	if(mc->mail_timeout != 0) {
-		gtk_timeout_remove(mc->mail_timeout);
-		mc->mail_timeout = 0;
-	}
-
+	
 	switch (error) {
 	case NETWORK_ERROR:
-		details = _("The Network may be down.");
+		details = _("Could not connect to the Internet.");
 		break;
 	case INVALID_USER:
-		details = _("The User id or Password is not correct.");
-		break;
 	case INVALID_PASS:
-		details = _("The User id or Password is not correct.");
+		details = _("The username or password is incorrect.");
 		break;	
 	case INVALID_SERVER:
-		details = _("The Server name may be wrong.");
+		details = _("The server name is incorrect.");
 		break;
 	case NO_SERVER_INFO:
 	default:
-		details = _("Maybe you used a wrong server, username or password?");
+		details = _("Error connecting to mail server.");
 		break;
 	}
-	
-	/* Notify about an error and keep the current mail status */
-	dialog = gtk_message_dialog_new (NULL,
-					 0,/* Flags */
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("The Inbox Monitor failed to check your mails from %s and thus automatic updating has been deactivated for now.\n%s"), mc->remote_server, details);
-
-	gtk_window_set_screen (GTK_WINDOW (dialog),
-			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
-
-	g_signal_connect_swapped (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  dialog);
-	gtk_widget_show_all (dialog);
+	/* FIXME: Show some sort of error images for non text mode */
+	if (mc->report_mail_mode == REPORT_MAIL_USE_TEXT) {
+		gtk_label_set_text (GTK_LABEL (mc->label), _("Error"));
+	}
+		
+	set_tooltip (GTK_WIDGET (mc->applet), details);
+	if (mc->animation_tag != 0){
+		gtk_timeout_remove (mc->animation_tag);
+		mc->animation_tag = 0;
+	}
 }
 
 static void
@@ -608,7 +922,7 @@ after_mail_check (MailCheck *mc)
 		}
 		else {
 			if(mc->unreadmail)
-				text = g_strdup_printf(_("%d/%d messages"), mc->unreadmail, mc->totalmail);
+				text = g_strdup_printf(_("%d unread/ %d messages"), mc->unreadmail, mc->totalmail);
 			else
 				text = g_strdup_printf(_("%d messages"), mc->totalmail);
 		} 
@@ -617,6 +931,8 @@ after_mail_check (MailCheck *mc)
 		text = g_strdup_printf(_("No mail."));
 
 	if (mc->newmail) {
+		mc->show_animation = TRUE;
+		
 		if(mc->play_sound)
 			gnome_triggers_vdo("You've got new mail!", "program", supinfo);
 
@@ -630,7 +946,7 @@ after_mail_check (MailCheck *mc)
 	case REPORT_MAIL_USE_ANIMATION:
 		if (mc->anymail){
 			if (mc->unreadmail){
-				if (mc->animation_tag == 0){
+				if (mc->animation_tag == 0 && mc->show_animation){
 					mc->animation_tag = gtk_timeout_add (150, next_frame, mc);
 					mc->nframe = 1;
 				}
@@ -730,13 +1046,17 @@ exec_clicked_cmd (GtkWidget *widget, GdkEventButton *event, gpointer data)
 	gboolean retval = FALSE;
 
 	if (event->button == 1) {
-		
+
 		if (mc->clicked_enabled && mc->clicked_cmd && (strlen(mc->clicked_cmd) > 0))
 			mailcheck_execute_shell (mc, mc->clicked_cmd);
-		
+
 		if (mc->reset_on_clicked) {
-			mc->newmail = mc->unreadmail = 0;
-			after_mail_check (mc);
+	
+			mc->show_animation = FALSE;
+			if (mc->animation_tag != 0){
+				gtk_timeout_remove (mc->animation_tag);
+				mc->animation_tag = 0;
+			}
 		}
 
 		retval = TRUE;
@@ -765,6 +1085,7 @@ mailcheck_destroy (GtkWidget *widget, gpointer data)
 	g_free (mc->remote_server);
 	g_free (mc->remote_username);
 	g_free (mc->remote_password);
+	g_free (mc->remote_encrypted_password);
 	g_free (mc->remote_folder);
 	g_free (mc->real_password);
 
@@ -1001,6 +1322,7 @@ make_remote_widgets_sensitive(MailCheck *mc)
 	
 	gtk_widget_set_sensitive (mc->remote_server_entry, b);
 	gtk_widget_set_sensitive (mc->remote_password_entry, b);
+	gtk_widget_set_sensitive (mc->remote_password_checkbox, b);
 	gtk_widget_set_sensitive (mc->remote_username_entry, b);
         gtk_widget_set_sensitive (mc->remote_folder_entry, f);
 	gtk_widget_set_sensitive (mc->remote_server_label, b);
@@ -1067,19 +1389,36 @@ remote_password_changed (GtkEntry *entry, gpointer data)
 {
 	MailCheck *mc = data;
 	gchar *text;
-	
+
 	text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 	if (!text)
 		return;
 		
-	if (mc->remote_password);
-		g_free (mc->remote_password);
-		
-	mc->remote_password = text;
-	panel_applet_gconf_set_string(mc->applet, "remote_password", 
-				      mc->remote_password, NULL);
+	if (mc->remote_password)
+		g_free (mc->remote_password); 
+	mc->remote_password = text; 
+
+	if (mc->remote_encrypted_password)
+		g_free (mc->remote_encrypted_password);
+	mc->remote_encrypted_password = util_base64 (text, ENCODE);  
+
+	if (mc->save_remote_password)
+		panel_applet_gconf_set_string (mc->applet, "remote_encrypted_password", 
+				      mc->remote_encrypted_password, NULL);
+	else
+		panel_applet_gconf_set_string (mc->applet, "remote_encrypted_password",
+				      "", NULL);
 }
 
+static gboolean
+focus_out_cb (GtkWidget *entry, GdkEventFocus *event, gpointer data)
+{
+	remote_password_changed (GTK_ENTRY (entry), data);
+
+	return FALSE;
+
+} 
+
 static void
 remote_folder_changed (GtkEntry *entry, gpointer data)
 {
@@ -1129,6 +1468,23 @@ pre_check_toggled (GtkToggleButton *button, gpointer data)
 
 }
 
+static void
+remote_password_save_toggled (GtkToggleButton *button, gpointer data)
+{
+	MailCheck *mc = data;
+
+	mc->save_remote_password = gtk_toggle_button_get_active (button);
+	panel_applet_gconf_set_bool (mc->applet, "save_password",
+				     mc->save_remote_password, NULL);
+
+	if (mc->save_remote_password)
+                panel_applet_gconf_set_string (mc->applet, "remote_encrypted_password",
+                                      mc->remote_encrypted_password, NULL);
+        else
+                panel_applet_gconf_set_string (mc->applet, "remote_encrypted_password",
+                                      "", NULL);
+}
+
 static void
 pre_check_changed (GtkEntry *entry, gpointer data)
 {
@@ -1252,9 +1608,9 @@ update_spin_changed (GtkSpinButton *spin, gpointer data)
 	mc->update_freq = 1000 * (guint)(gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mc->sec_spin)) + 60 * gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (mc->min_spin)));
 	
 	if (mc->update_freq == 0) {
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON (mc->sec_spin), 0.0);
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON (mc->min_spin), 1.0);
-		mc->update_freq = 60*1000;
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON (mc->sec_spin), 1.0);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON (mc->min_spin), 0.0);
+		mc->update_freq = 1000;
 	}
 	if(mc->mail_timeout != 0)
 		gtk_timeout_remove (mc->mail_timeout);
@@ -1462,8 +1818,29 @@ mailbox_properties_page(MailCheck *mc)
 	gtk_widget_show(l);
 	gtk_box_pack_start (GTK_BOX (control_hbox), l, TRUE, TRUE, 0);      
 	
-	g_signal_connect(G_OBJECT(l), "changed",
-                     G_CALLBACK(remote_password_changed), mc);
+	g_signal_connect(G_OBJECT(l), "focus_out_event",
+                     G_CALLBACK(focus_out_cb), mc);
+
+	g_signal_connect(G_OBJECT(l), "activate",
+                     G_CALLBACK(remote_password_changed), mc);  
+
+	control_hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (control_vbox), control_hbox, TRUE, TRUE, 0);
+	gtk_widget_show (control_hbox);
+
+	l = gtk_label_new ("");
+	gtk_size_group_add_widget (size_group, l);
+	gtk_widget_show (l);
+	gtk_box_pack_start (GTK_BOX (control_hbox), l, FALSE, FALSE, 0);
+
+	mc->remote_password_checkbox = l = gtk_check_button_new_with_mnemonic (
+					_("_Save password to disk"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(l), mc->save_remote_password);
+	gtk_widget_show (l);
+	gtk_box_pack_start (GTK_BOX (control_hbox), l, TRUE, TRUE, 0);
+	
+	g_signal_connect (G_OBJECT (l), "toggled", 
+			  G_CALLBACK(remote_password_save_toggled), mc);
 
 	control_hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (control_vbox), control_hbox, TRUE, TRUE, 0);
@@ -1496,7 +1873,7 @@ mailbox_properties_page(MailCheck *mc)
 static GtkWidget *
 mailcheck_properties_page (MailCheck *mc)
 {
-	GtkWidget *vbox, *hbox, *l, *table, *check_box, *animation_option_menu;
+	GtkWidget *vbox, *hbox, *hbox2, *l, *table, *check_box, *animation_option_menu;
 	GtkWidget *label, *indent, *categories_vbox, *category_vbox, *control_vbox, *control_hbox;
 	GtkObject *freq_a;
 	gchar *title;
@@ -1548,11 +1925,15 @@ mailcheck_properties_page (MailCheck *mc)
 	gtk_box_pack_start (GTK_BOX (control_hbox), l, FALSE, FALSE, 0);
 	gtk_widget_show(l);
 
+        hbox2 = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(control_hbox), hbox2, FALSE, FALSE, 0);
+	gtk_widget_show(hbox2);
+	
 	freq_a = gtk_adjustment_new((float)((mc->update_freq/1000)/60), 0, 1440, 1, 5, 5);
 	mc->min_spin = gtk_spin_button_new( GTK_ADJUSTMENT (freq_a), 1, 0);
 	g_signal_connect (G_OBJECT (mc->min_spin), "value_changed",
 			  G_CALLBACK (update_spin_changed), mc);			  
-	gtk_box_pack_start (GTK_BOX (control_hbox), mc->min_spin,  FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), mc->min_spin,  FALSE, FALSE, 0);
 	set_atk_name_description (mc->min_spin, _("minutes"), _("Choose time interval in minutes to check mail"));
 	set_atk_relation (mc->min_spin, check_box, ATK_RELATION_CONTROLLED_BY);
 	gtk_widget_show(mc->min_spin);
@@ -1560,13 +1941,17 @@ mailcheck_properties_page (MailCheck *mc)
 	l = gtk_label_new (_("minutes"));
 	set_atk_relation (mc->min_spin, l, ATK_RELATION_LABELLED_BY);
 	gtk_widget_show(l);
-	gtk_box_pack_start (GTK_BOX (control_hbox), l, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), l, FALSE, FALSE, 0);
 
+        hbox2 = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(control_hbox), hbox2, FALSE, FALSE, 0);
+	gtk_widget_show(hbox2);
+	
 	freq_a = gtk_adjustment_new((float)((mc->update_freq/1000)%60), 0, 59, 1, 5, 5);
 	mc->sec_spin = gtk_spin_button_new (GTK_ADJUSTMENT (freq_a), 1, 0);
 	g_signal_connect (G_OBJECT (mc->sec_spin), "value_changed",
 			  G_CALLBACK (update_spin_changed), mc);
-	gtk_box_pack_start (GTK_BOX (control_hbox), mc->sec_spin,  FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), mc->sec_spin,  FALSE, FALSE, 0);
 	set_atk_name_description (mc->sec_spin, _("seconds"), _("Choose time interval in seconds to check mail"));
 	set_atk_relation (mc->sec_spin, check_box, ATK_RELATION_CONTROLLED_BY);
 	gtk_widget_show(mc->sec_spin);
@@ -1575,7 +1960,7 @@ mailcheck_properties_page (MailCheck *mc)
 	set_atk_relation (mc->sec_spin, l,  ATK_RELATION_LABELLED_BY);
 	gtk_widget_show(l);
 	gtk_misc_set_alignment (GTK_MISC (l), 0.0f, 0.5f);
-	gtk_box_pack_start (GTK_BOX (control_hbox), l, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), l, TRUE, TRUE, 0);
 
 	set_atk_relation (check_box, mc->min_spin, ATK_RELATION_CONTROLLER_FOR);
 	set_atk_relation (check_box, mc->sec_spin, ATK_RELATION_CONTROLLER_FOR);
@@ -1587,7 +1972,8 @@ mailcheck_properties_page (MailCheck *mc)
 	gtk_widget_show(mc->play_sound_check);
 	gtk_box_pack_start(GTK_BOX (control_vbox), mc->play_sound_check, TRUE, TRUE, 0);
 
-	l = gtk_check_button_new_with_mnemonic (_("Set the number of unread mails to _zero when clicked"));
+	/*l = gtk_check_button_new_with_mnemonic (_("Set the number of unread mails to _zero when clicked"));*/
+	l = gtk_check_button_new_with_mnemonic (_("Sto_p animation when clicked"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l), mc->reset_on_clicked);
 	g_signal_connect(G_OBJECT(l), "toggled",
 			   G_CALLBACK(reset_on_clicked_toggled), mc);
@@ -1606,7 +1992,7 @@ mailcheck_properties_page (MailCheck *mc)
 	gtk_box_pack_start (GTK_BOX (hbox), animation_option_menu, FALSE, FALSE, 0);
 	set_atk_relation (animation_option_menu, l, ATK_RELATION_LABELLED_BY);
 	make_check_widgets_sensitive(mc);
-
+	
 	category_vbox = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (categories_vbox), category_vbox, TRUE, TRUE, 0);
 	gtk_widget_show (category_vbox);
@@ -1737,7 +2123,7 @@ phelp_cb (GtkDialog *w, gint tab, MailCheck *mc)
 		dialog = gtk_message_dialog_new (GTK_WINDOW (w),
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						  _("There was an error displaying help: %s"),
 						 error->message);
 
@@ -1792,13 +2178,14 @@ mailcheck_properties (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbna
 			       gtk_widget_get_screen (GTK_WIDGET (mc->property_window)));
 	gtk_dialog_set_default_response (GTK_DIALOG (mc->property_window), GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (mc->property_window), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (mc->property_window), 5);
 	gnome_window_icon_set_from_file (GTK_WINDOW (mc->property_window),
 					 GNOME_ICONDIR"/gnome-mailcheck.png");
 	gtk_window_set_screen (GTK_WINDOW (mc->property_window),
 			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 	
 	notebook = gtk_notebook_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (notebook), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
 	gtk_widget_show (notebook);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (mc->property_window)->vbox), notebook,
 			    TRUE, TRUE, 0);
@@ -1827,12 +2214,10 @@ check_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 static void
 applet_load_prefs(MailCheck *mc)
 {
-	mc->animation_file = panel_applet_gconf_get_string(mc->applet, "animation_file", NULL);
-	if(!mc->animation_file) {
-		g_free(mc->animation_file);
-		mc->animation_file = NULL;
-	}
+	panel_applet_gconf_set_string(mc->applet, "remote_password", "", NULL);
 
+	mc->animation_file = panel_applet_gconf_get_string(mc->applet, "animation_file", NULL);
+	
 	mc->auto_update = panel_applet_gconf_get_bool(mc->applet, "auto_update", NULL);
 	mc->reset_on_clicked = panel_applet_gconf_get_bool (mc->applet, "reset_on_clicked", NULL);
 	mc->update_freq = panel_applet_gconf_get_int(mc->applet, "update_frequency", NULL);
@@ -1848,7 +2233,10 @@ applet_load_prefs(MailCheck *mc)
 		g_free(mc->remote_username);
 		mc->remote_username = g_strdup(g_getenv("USER"));
 	}
-	mc->remote_password = panel_applet_gconf_get_string(mc->applet, "remote_password", NULL);
+	mc->remote_encrypted_password = panel_applet_gconf_get_string (mc->applet, 
+					"remote_encrypted_password", NULL); 
+	mc->remote_password = util_base64 (mc->remote_encrypted_password, DECODE);
+	mc->save_remote_password = panel_applet_gconf_get_bool (mc->applet, "save_password", NULL);
 	mc->remote_folder = panel_applet_gconf_get_string(mc->applet, "remote_folder", NULL);
 	mc->mailbox_type = panel_applet_gconf_get_int(mc->applet, "mailbox_type", NULL);
 	mc->mail_file = panel_applet_gconf_get_string (mc->applet, "mail_file", NULL);
@@ -1949,7 +2337,7 @@ help_callback (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						  _("There was an error displaying help: %s"),
 						 error->message);
 
@@ -2031,28 +2419,26 @@ mailcheck_applet_fill (PanelApplet *applet)
 	mc->animation_tag = 0;
 	mc->password_dialog = NULL;
 	mc->oldsize = 0;
+	mc->show_animation = TRUE;
 
 	/*initial state*/
 	mc->report_mail_mode = REPORT_MAIL_USE_ANIMATION;
 	
-	mc->mail_file = NULL;
+	mc->mail_file = NULL;	
 
-	if (mc->mail_file == NULL) {
+	panel_applet_add_preferences (applet, "/schemas/apps/mailcheck_applet/prefs", NULL);
+	applet_load_prefs(mc);
+	if (mc->mail_file == NULL || strlen(mc->mail_file)==0) {
 		const char *mail_file = g_getenv ("MAIL");
 		if (mail_file == NULL) {
 			const char *user = g_getenv ("USER");
-			if (user == NULL)
-				return FALSE;
-
-			mc->mail_file = g_strdup_printf ("/var/spool/mail/%s",
+			if (user != NULL)
+				mc->mail_file = g_strdup_printf ("/var/spool/mail/%s",
 							 user);
 		} else
 			mc->mail_file = g_strdup (mail_file);
 	}
 
-	panel_applet_add_preferences (applet, "/schemas/apps/mailcheck_applet/prefs", NULL);
-	applet_load_prefs(mc);
-
 	mc->mailcheck_text_only = _("Text only");
 
 	mc->size = panel_applet_get_size (applet);
